﻿-- 
-- ACP - Addon Control Panel (Formerly Master Control Program)
--
-- Allows you to control AddOn loading after logging in.
-- 
-- Personal Use Version by Sylvaan - Proudmoore US/Alliance
--
--

ACP_SelectedProfile = "NONE";

UIPanelWindows["ACP_AddonList"] = { area = "center", pushable = 0, whileDead = 1 };
StaticPopupDialogs["ACP_RELOADUI"] = {
	text = ACP_RELOAD,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	OnAccept = function()
		ReloadUI();
	end,
	timeout = 0,
	hideOnEscape = 1
};

function ACP_DeleteDialog()
  StaticPopupDialogs["ACP_DELETEPROFILE"] = {
	text = ACP_DELETE_PROFILE_DIALOG..ACP_SelectedProfile,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	OnAccept = function()
		ACP_DeleteProfile(ACP_SelectedProfile);
                ACP_SelectedProfile = "NONE";
                UIDropDownMenu_SetText(ACP_SelectedProfile, ACP_AddonList_ProfileSelection);
        end,
	timeout = 0,
	hideOnEscape = 1
  };
  StaticPopup_Show("ACP_DELETEPROFILE");
end

function ACP_SaveDialog()
  StaticPopupDialogs["ACP_SAVEPROFILE"] = {
        text = ACP_PROFILE_NAME_SAVE;
        button1 = TEXT(ACCEPT),
        button2 = TEXT(CANCEL),
        hasEditBox = 1,
        maxLetters = 32,
        hideOnEscape = 1,
        OnLoad = function()
                getglobal(this:GetParent():GetName().."EditBox"):SetText(ACP_SelectedProfile);
        end,
        OnAccept = function()
                ACP_SaveProfile(getglobal(this:GetParent():GetName().."EditBox"):GetText());
        end,
        timeout = 0,
        EditBoxOnEnterPressed = function()
                ACP_SaveProfile(getglobal(this:GetParent():GetName().."EditBox"):GetText());
                this:GetParent():Hide();
        end
  };
  StaticPopup_Show("ACP_SAVEPROFILE");
end

ACP_VERSION = "2006.09.06";
ACP_LINEHEIGHT = 16;
local ACP_MAXADDONS = 20;
local ACP_BLIZZARD_ADDONS = { 
    "Blizzard_AuctionUI",
    "Blizzard_BattlefieldMinimap",
    "Blizzard_BindingUI",
    "Blizzard_CombatText",
    "Blizzard_CraftUI",
    "Blizzard_GMSurveyUI",
    "Blizzard_InspectUI",
    "Blizzard_ItemSocketingUI",
    "Blizzard_MacroUI",
    "Blizzard_RaidUI",
    "Blizzard_TalentUI",
    "Blizzard_TradeSkillUI",
    "Blizzard_TrainerUI",
};
local ACP_BLIZZARD_ADDONS_TITLES = { 
    "Blizzard: Auction",
    "Blizzard: Battlefield Minimap",
    "Blizzard: Binding",
    "Blizzard: Combat Text",
    "Blizzard: Craft",
    "Blizzard: GM Survey",
    "Blizzard: Inspect",
    "Blizzard: Item Socketing",
    "Blizzard: Macro",
    "Blizzard: Raid",
    "Blizzard: Talent",
    "Blizzard: Trade Skill",
    "Blizzard: Trainer",
};
local ACP_old_LoadAddOn = nil;


local function ACP_new_LoadAddOn(name)
	if (not IsAddOnLoaded(name) and ACP_Config and ACP_Config.refusetoload and ACP_Config.refusetoload[name]) then
		return nil, "REFUSE_TO_LOAD";
	else
		return ACP_old_LoadAddOn(name);
	end
end

function ACP_OnLoad()
	ACP_old_LoadAddOn = LoadAddOn;
--	hooksecurefunc("LoadAddOn", ACP_new_LoadAddOn)

        -- setup /ACP slash command
        SlashCmdList["ACPSLASHCMD"] = ACP_SlashHandler;
        SLASH_ACPSLASHCMD1 = "/ACP";
end
-- /ACP slash command, only shows the UI
function ACP_SlashHandler(msg)
        ShowUIPanel(ACP_AddonList);
end

function ACP_AddonList_Enable(index,enabled)
	if (type(index) == "number") then
		if (enabled) then
            ACP_EnableRecurse(index)
--			EnableAddOn(index)
		else
			DisableAddOn(index)
		end
	else
		if (enabled) then
			if (ACP_Config and ACP_Config.refusetoload) then
				ACP_Config.refusetoload[index] = nil;
			end
		else
			if (not ACP_Config) then ACP_Config = {}; end
			if (not ACP_Config.refusetoload) then ACP_Config.refusetoload = {}; end
			ACP_Config.refusetoload[index] = true;
		end
	end
	ACP_AddonList_OnShow();
end

function ACP_AddonList_LoadNow(index)
	UIParentLoadAddOn(index);
	ACP_AddonList_OnShow();
end

function ACP_AddonList_OnShow()
	local function setSecurity (obj, idx)
		local width,height,iconWidth = 64,16,16;
		local increment = iconWidth/width;
		local left = (idx-1)*increment;
		local right = idx*increment;
		obj:SetTexCoord(left, right, 0, 1);
	end

	local numAddons = GetNumAddOns();
	local origNumAddons = numAddons;
	numAddons = numAddons + #(ACP_BLIZZARD_ADDONS);
	FauxScrollFrame_Update(ACP_AddonList_ScrollFrame, numAddons, ACP_MAXADDONS, ACP_LINEHEIGHT, nil, nil, nil);
	local i;
	local offset = FauxScrollFrame_GetOffset(ACP_AddonList_ScrollFrame);
	for i = 1, ACP_MAXADDONS, 1 do
		obj = getglobal("ACP_AddonListEntry"..i);
		local addonIdx = offset+i;
		if (addonIdx > numAddons) then
			obj:Hide();
			obj.addon = nil;
		else
			obj:Show();
			local titleText = getglobal("ACP_AddonListEntry"..i.."Title");
			local status = getglobal("ACP_AddonListEntry"..i.."Status");
			local checkbox = getglobal("ACP_AddonListEntry"..i.."Enabled");
			local securityIcon = getglobal("ACP_AddonListEntry"..i.."SecurityIcon");
			local loadnow = getglobal("ACP_AddonListEntry"..i.."LoadNow");

			local name, title, notes, enabled, loadable, reason, security;
			if (addonIdx > origNumAddons) then
				name = ACP_BLIZZARD_ADDONS[(addonIdx-origNumAddons)];
				obj.addon = name;
				title = ACP_BLIZZARD_ADDONS_TITLES[(addonIdx-origNumAddons)];
				notes = "";
				if (ACP_Config and ACP_Config.refusetoload and ACP_Config.refusetoload[name]) then
					enabled = nil;
					loadable = nil;
					reason = "WILL_NOT_LOAD";
				else
					enabled = 1;
					loadable = 1;
				end
				if (IsAddOnLoaded(name)) then
					reason = "LOADED";
					loadable = 1;
				end
				security = "SECURE";
			else
				name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(addonIdx);
				obj.addon = addonIdx;
			end
			local loaded = IsAddOnLoaded(name);
			local ondemand = IsAddOnLoadOnDemand(name);
			if (loadable) then
				titleText:SetTextColor(1,0.78,0);
			elseif (enabled and reason ~= "DEP_DISABLED") then
				titleText:SetTextColor(1,0.1,0.1);
			else
				titleText:SetTextColor(0.5,0.5,0.5);
			end
			if (title) then
				titleText:SetText(title);
			else
				titleText:SetText(name);
			end
			if (title == "Addon Control Panel")  then
				checkbox:Hide();
            else
				checkbox:Show();
				checkbox:SetChecked(enabled);
			end
			if (security == "SECURE") then
				setSecurity(securityIcon,1);
			elseif (security == "INSECURE") then
				setSecurity(securityIcon,2);
			elseif (security == "BANNED") then
				setSecurity(securityIcon,3);
			end
			if (reason) then
				status:SetText(TEXT(getglobal("ACP_ADDON_"..reason)));
			elseif (loaded) then
				status:SetText(TEXT(ACP_ADDON_LOADED));
			elseif (ondemand) then
				status:SetText(ACP_LOADED_ON_DEMAND);
			else
				status:SetText("");
			end
			if (not loaded and enabled and ondemand) then
				loadnow:Show();
			else
				loadnow:Hide();
			end
		end

	end
end



function ACP_SaveProfile(profile)
  if profile == "NONE" then return end

  local numAddons = GetNumAddOns();
  local i;

  -- make sure we have a config for profile setup
  if not ACP_Config then ACP_Config = {} end
  if not ACP_Config.profiles then ACP_Config.profiles = {} end

  -- setup this profile, and clear any previous data in case it exists
  ACP_Config.profiles[profile] = {};

  -- set each addon to be loaded or not loaded
  for i = 1, numAddons do
    local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(i);
    if enabled then
      ACP_Config.profiles[profile][name] = 1;
    else
      ACP_Config.profiles[profile][name] = 0;
    end
  end


  StaticPopupDialogs["ACP_PROFILESAVED"] = {
    text=ACP_PROFILE_SAVED..profile,
    button1 = TEXT(OK),
    OnAccept = function()
    end,
    timeout = 2,
    hideOnEscape = 1
  };

  StaticPopup_Show("ACP_PROFILESAVED");
end

function ACP_LoadProfile()
  local i;
  local addons;

  ACP_SelectedProfile = this.value;
  if not ACP_SelectedProfile or ACP_SelectedProfile == "" then
    ACP_SelectedProfile = "NONE";
    return;
  end


  for addons in ACP_Config.profiles[ACP_SelectedProfile] do
    if ACP_Config.profiles[ACP_SelectedProfile][addons] == 1 then
      EnableAddOn(addons);
    else
      DisableAddOn(addons);
    end
  end


  if not ACP_Config then ACP_Config = {}; end
  ACP_Config.SelectedProfile = ACP_SelectedProfile;
  UIDropDownMenu_SetText(ACP_SelectedProfile, ACP_AddonList_ProfileSelection);

  ACP_AddonList_OnShow()
end


function ACP_DeleteProfile(profile)
  if not ACP_Config then ACP_Config = {}; end
  if not ACP_Config.profiles then ACP_Config.profiles = {}; end
  local buttontext = ACP_PROFILE_DELETED..profile;

  if profile == "NONE" then
    buttontext = ACP_NO_PROFILE_DELETED;
  else
    ACP_Config.profiles[profile] = nil;
  end

  ACP_DeleteDialog();
end


function ACP_ResetProfiles(param)
  UIDropDownMenu_Initialize(this,  ACP_InitializeProfiles);
  UIDropDownMenu_SetWidth(220, this);

  if ACP_Config and ACP_Config.SelectedProfile then
    ACP_SelectedProfile = ACP_Config.SelectedProfile;
    UIDropDownMenu_SetText(ACP_SelectedProfile, ACP_AddonList_ProfileSelection);
  end
end

function ACP_InitializeProfiles()
  if ACP_Config and ACP_Config.profiles then
    local info = {};
    local profile;

    for profile in pairs(ACP_Config.profiles) do
      info = {
        ["text"] = profile,
        ["value"] = profile,
        ["func"] = ACP_LoadProfile,
      };
      UIDropDownMenu_AddButton(info);
    end
  end
end


function ACP_EnableAll()
  local numAddons = GetNumAddOns();
  local i;

  -- set each addon to be enabled
  for i = 1, numAddons do
    local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(i);
    if not enabled then
      EnableAddOn(name);
    end
  end
  ACP_AddonList_OnShow()
end

function ACP_DisableAll()
  local numAddons = GetNumAddOns();
  local i;

  -- set each addon to be enabled
  for i = 1, numAddons do
    local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(i);
    if enabled and title ~= "Addon Control Panel" then
      DisableAddOn(name);
    end
  end
  ACP_AddonList_OnShow()
end


local function stripSpaces(text)
	if type(text) == "string" then
		return strtrim(text)
	end
	return text
end

MC = {}
function ACP_TooltipShow(index)
  local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(index);

  local version = stripSpaces(GetAddOnMetadata(name, "Version"))

  GameTooltip:SetOwner(this, "ANCHOR_BOTTOMLEFT");

  if version then
	if string.find(version, "%$Revision: (%d+) %$") then
		version = string.gsub(version, "%$Revision: (%d+) %$", "%1")
	elseif string.find(version, "%$Rev: (%d+) %$") then
		version = string.gsub(version, "%$Rev: (%d+) %$", "%1")
	elseif string.find(version, "%$LastChangedRevision: (%d+) %$") then
		version = string.gsub(version, "%$LastChangedRevision: (%d+) %$", "%1")
	end
  end

	local x
	if title then
		x = "|cffffff7f" .. tostring(title) .. "|r"
	elseif name then
		x = "|cffffff7f" .. tostring(name) .. "|r"
	else
		x = "|cffffff7f<" .. tostring(class) .. " instance>|r"
	end
	if version then
		x = x .. " - |cffffff7f" .. tostring(version) .. "|r"
	end
	GameTooltip:AddLine(x)
	
	
	local author, credits, ldate
	
	if author == nil then
		author = GetAddOnMetadata(name, "Author")
		author = stripSpaces(author)
	end
	if credits == nil then
		credits = GetAddOnMetadata(name, "X-Credits")
		credits = stripSpaces(credits)
	end
	if ldate == nil then
		ldate = GetAddOnMetadata(name, "X-Date") or GetAddOnMetadata(name, "X-ReleaseDate")
	end
	if ldate then
		if string.find(ldate, "%$Date: (.-) %$") then
			ldate = string.gsub(ldate, "%$Date: (.-) %$", "%1")
		elseif string.find(ldate, "%$LastChangedDate: (.-) %$") then
			ldate = string.gsub(ldate, "%$LastChangedDate: (.-) %$", "%1")
		end
	end
	ldate = stripSpaces(ldate)


	if notes then
		GameTooltip:AddLine("|cffffff7f" .. tostring(notes) .. "|r")
	end
	if author then
		GameTooltip:AddLine(" - |cffffff7f" .. "Author" .. ":|r " .. tostring(author))
	end
	if credits then
		GameTooltip:AddLine(" - |cffffff7f" .. "Credits" .. ":|r " .. tostring(credits))
	end
	if ldate then
		GameTooltip:AddLine(" - |cffffff7f" .. "Date" .. ":|r " .. tostring(ldate))
	end


	local category = stripSpaces(GetAddOnMetadata(name, "X-Category")) 
	if category then
		GameTooltip:AddLine(" - |cffffff7f" .. "Category" .. ":|r " .. category)
	end

    local RequiredDeps = build_string(GetAddOnDependencies(index))
	if RequiredDeps then
		GameTooltip:AddLine(" - |cffffff7f" .. "Dependancies" .. ":|r " .. RequiredDeps)
	end

    local metaXEmbeds = stripSpaces(GetAddOnMetadata(name, "X-Embeds"))
	if metaXEmbeds then
		GameTooltip:AddLine(" - |cffffff7f" .. "Embeds" .. ":|r " .. metaXEmbeds)
	end

    if reason then 
		GameTooltip:AddLine(" - |cffff7f7f" .. "Reason" .. ":|r " .. tostring(getglobal("ADDON_"..reason)))
	end


  GameTooltip:Show();
end

function build_string(...)
        local s 
        for i=1,select("#", ...) do
        local x = select(i, ...)
        if x and strlen(x) then
            s = s and (s..", "..x) or x
        end
      end
    return s
end




function ACP_EnableRecurse(name)
    EnableAddOn(name)

    recursive_iterate_over(GetAddOnDependencies(name))

    local metaXEmbeds = stripSpaces(GetAddOnMetadata(name, "X-Embeds"))
    if metaXEmbeds then
        recursive_iterate_over(strsplit(" ,", metaXEmbeds))
    end
end


function iterate_over(...)
  for i=1,select("#", ...) do
    local x = select(i, ...)
    if x and strlen(x) then
        EnableAddOn(x)
    end
  end
end

function recursive_iterate_over(...)
  for i=1,select("#", ...) do
    local x = select(i, ...)
    if x and strlen(x) then
        ACP_EnableRecurse(x)
    end
  end
end






function ACP_PrintAddonInfo()
	local x
	if title then
		x = "|cffffff7f" .. tostring(self.title) .. "|r"
	elseif name then
		x = "|cffffff7f" .. tostring(name) .. "|r"
	else
		x = "|cffffff7f<" .. tostring(class) .. " instance>|r"
	end
	if version then
		x = x .. " - |cffffff7f" .. tostring(version) .. "|r"
	end
	if notes then
		x = x .. " - " .. tostring(notes)
	end
	GameTooltip:AddLine(x)
	if author then
		print(" - |cffffff7f" .. AUTHOR .. ":|r " .. tostring(author))
	end
	if credits then
		print(" - |cffffff7f" .. CREDITS .. ":|r " .. tostring(credits))
	end
	if date then
		print(" - |cffffff7f" .. DATE .. ":|r " .. tostring(date))
	end
	if category then
		local category = CATEGORIES[category]
		if category then
			print(" - |cffffff7f" .. CATEGORY .. ":|r " .. category)
		end
	end
	if email then
		print(" - |cffffff7f" .. EMAIL .. ":|r " .. tostring(email))
	end
	if website then
		print(" - |cffffff7f" .. WEBSITE .. ":|r " .. tostring(website))
	end
end

